@@ -6,9 +6,9 @@ use crate::bbox::BoundingBox;
 use crate::coord_units::CoordUnits;
 use crate::document::AcquiredNodes;
 use crate::drawing_ctx::{DrawingCtx, ViewParams};
-use crate::filter::Filter;
-use crate::node::RsvgNode;
+use crate::node::{Node, NodeBorrow};
 use crate::paint_server::PaintServer;
+use crate::parsers::CustomIdent;
 use crate::properties::ComputedValues;
 use crate::rect::IRect;
 use crate::surface_utils::shared_surface::{
@@ -18,7 +18,7 @@ use crate::transform::Transform;
 use crate::unit_interval::UnitInterval;
 
 use super::error::FilterError;
-use super::input::{CustomIdent, Input};
+use super::Input;
 
 /// A filter primitive output.
 #[derive(Debug, Clone)]
@@ -52,7 +52,7 @@ pub enum FilterInput {
 /// The filter rendering context.
 pub struct FilterContext {
     /// The <filter> node.
-    node: RsvgNode,
+    node: Node,
     /// Bounding box of node being filtered
     node_bbox: BoundingBox,
     /// Values from the node which referenced this filter.
@@ -99,7 +99,7 @@ pub struct FilterContext {
 impl FilterContext {
     /// Creates a new `FilterContext`.
     pub fn new(
-        filter_node: &RsvgNode,
+        filter_node: &Node,
         computed_from_node_being_filtered: &ComputedValues,
         source_surface: SharedImageSurface,
         draw_ctx: &mut DrawingCtx,
@@ -111,8 +111,7 @@ impl FilterContext {
         // However, with userSpaceOnUse it's still possible to create images with a filter.
         let bbox_rect = node_bbox.rect.unwrap_or_default();
 
-        let node_data = filter_node.borrow();
-        let filter = node_data.get_impl::<Filter>();
+        let filter = borrow_element_as!(filter_node, Filter);
 
         let affine = match filter.get_filter_units() {
             CoordUnits::UserSpaceOnUse => draw_transform,
@@ -256,11 +255,9 @@ impl FilterContext {
 
     /// Pushes the viewport size based on the value of `primitiveUnits`.
     pub fn get_view_params(&self, draw_ctx: &mut DrawingCtx) -> ViewParams {
-        let node_data = self.node.borrow();
-        let filter = node_data.get_impl::<Filter>();
-
         // See comments in compute_effects_region() for how this works.
-        if filter.get_primitive_units() == CoordUnits::ObjectBoundingBox {
+        let units = borrow_element_as!(self.node, Filter).get_primitive_units();
+        if units == CoordUnits::ObjectBoundingBox {
             draw_ctx.push_view_box(1.0, 1.0)
         } else {
             draw_ctx.get_view_params()
@@ -292,7 +289,7 @@ impl FilterContext {
                     paint_server,
                     opacity,
                     &self.node_bbox,
-                    self.computed_from_node_being_filtered.color.0,
+                    self.computed_from_node_being_filtered.color().0,
                 )
                 .and_then(|had_paint_server| {
                     if had_paint_server {
@@ -357,8 +354,8 @@ impl FilterContext {
                 .get_paint_server_surface(
                     draw_ctx,
                     acquired_nodes,
-                    &values.fill.0,
-                    values.fill_opacity.0,
+                    &values.fill().0,
+                    values.fill_opacity().0,
                 )
                 .map_err(FilterError::CairoError)
                 .map(FilterInput::StandardInput),
@@ -367,8 +364,8 @@ impl FilterContext {
                 .get_paint_server_surface(
                     draw_ctx,
                     acquired_nodes,
-                    &values.stroke.0,
-                    values.stroke_opacity.0,
+                    &values.stroke().0,
+                    values.stroke_opacity().0,
                 )
                 .map_err(FilterError::CairoError)
                 .map(FilterInput::StandardInput),
