@@ -2,9 +2,11 @@ use cssparser::Parser;
 use markup5ever::{expanded_name, local_name, namespace_url, ns, QualName};
 use nalgebra::{DMatrix, Dynamic, VecStorage};
 
+use crate::document::AcquiredNodes;
 use crate::drawing_ctx::DrawingCtx;
+use crate::element::{ElementResult, SetAttributes};
 use crate::error::*;
-use crate::node::{NodeResult, NodeTrait, RsvgNode};
+use crate::node::Node;
 use crate::number_list::{NumberList, NumberListLength};
 use crate::parsers::{NumberOptionalNumber, Parse, ParseValue};
 use crate::property_bag::PropertyBag;
@@ -52,11 +54,9 @@ impl Default for FeConvolveMatrix {
     }
 }
 
-impl NodeTrait for FeConvolveMatrix {
-    impl_node_as_filter_effect!();
-
-    fn set_atts(&mut self, parent: Option<&RsvgNode>, pbag: &PropertyBag<'_>) -> NodeResult {
-        self.base.set_atts(parent, pbag)?;
+impl SetAttributes for FeConvolveMatrix {
+    fn set_attributes(&mut self, pbag: &PropertyBag<'_>) -> ElementResult {
+        self.base.set_attributes(pbag)?;
 
         for (attr, value) in pbag.iter() {
             match attr.expanded() {
@@ -66,7 +66,7 @@ impl NodeTrait for FeConvolveMatrix {
                             if v.0 > 0 && v.1 > 0 {
                                 Ok(v)
                             } else {
-                                Err(ValueErrorKind::value_error("values must be greater than 0"))?
+                                Err(ValueErrorKind::value_error("values must be greater than 0"))
                             }
                         })?;
                     self.order = (x as u32, y as u32);
@@ -196,14 +196,15 @@ impl NodeTrait for FeConvolveMatrix {
 impl FilterEffect for FeConvolveMatrix {
     fn render(
         &self,
-        _node: &RsvgNode,
+        node: &Node,
         ctx: &FilterContext,
+        acquired_nodes: &mut AcquiredNodes,
         draw_ctx: &mut DrawingCtx,
     ) -> Result<FilterResult, FilterError> {
-        let input = self.base.get_input(ctx, draw_ctx)?;
+        let input = self.base.get_input(ctx, acquired_nodes, draw_ctx)?;
         let mut bounds = self
             .base
-            .get_bounds(ctx)
+            .get_bounds(ctx, node.parent().as_ref())?
             .add_input(&input)
             .into_irect(draw_ctx);
         let original_bounds = bounds;
