@@ -1,19 +1,19 @@
 //! Entry point for the CSS filters infrastructure.
 
+use cssparser::{BasicParseError, Parser};
+use markup5ever::{expanded_name, local_name, namespace_url, ns};
 use std::ops::Deref;
 use std::time::Instant;
 
-use markup5ever::{expanded_name, local_name, namespace_url, ns};
-
 use crate::bbox::BoundingBox;
 use crate::coord_units::CoordUnits;
 use crate::document::AcquiredNodes;
 use crate::drawing_ctx::DrawingCtx;
-use crate::error::{RenderingError, ValueErrorKind};
-use crate::filter::Filter;
+use crate::element::{Draw, Element, ElementResult, SetAttributes};
+use crate::error::{ParseError, RenderingError};
 use crate::length::*;
-use crate::node::{CascadedValues, NodeResult, NodeTrait, NodeType, RsvgNode};
-use crate::parsers::ParseValue;
+use crate::node::{CascadedValues, Node, NodeBorrow};
+use crate::parsers::{CustomIdent, Parse, ParseValue};
 use crate::properties::ComputedValues;
 use crate::property_bag::PropertyBag;
 use crate::property_defs::ColorInterpolationFilters;
@@ -28,18 +28,15 @@ use self::context::{FilterContext, FilterInput, FilterResult};
 mod error;
 use self::error::FilterError;
 
-mod input;
-use self::input::{CustomIdent, Input};
-
 /// A filter primitive interface.
-pub trait FilterEffect: NodeTrait {
+pub trait FilterEffect: SetAttributes + Draw {
     /// Renders this filter primitive.
     ///
     /// If this filter primitive can't be rendered for whatever reason (for instance, a required
     /// property hasn't been provided), an error is returned.
     fn render(
         &self,
-        node: &RsvgNode,
+        node: &Node,
         ctx: &FilterContext,
         acquired_nodes: &mut AcquiredNodes,
         draw_ctx: &mut DrawingCtx,
@@ -53,13 +50,8 @@ pub trait FilterEffect: NodeTrait {
     fn is_affected_by_color_interpolation_filters(&self) -> bool;
 }
 
-macro_rules! impl_node_as_filter_effect {
-    () => {
-        fn as_filter_effect(&self) -> Option<&dyn FilterEffect> {
-            Some(self)
-        }
-    };
-}
+// Filter Effects do not need to draw themselves
+impl<T: FilterEffect> Draw for T {}
 
 pub mod blend;
 pub mod color_matrix;
@@ -70,7 +62,7 @@ pub mod displacement_map;
 pub mod flood;
 pub mod gaussian_blur;
 pub mod image;
-pub mod light;
+pub mod lighting;
 pub mod merge;
 pub mod morphology;
 pub mod offset;
@@ -86,6 +78,39 @@ struct Primitive {
     result: Option<CustomIdent>,
 }
 
+/// An enumeration of possible inputs for a filter primitive.
+#[derive(Debug, Clone, Eq, PartialEq, Hash)]
+pub enum Input {
+    SourceGraphic,
+    SourceAlpha,
+    BackgroundImage,
+    BackgroundAlpha,
+    FillPaint,
+    StrokePaint,
+    FilterOutput(CustomIdent),
+}
+
+impl Parse for Input {
+    fn parse<'i>(parser: &mut Parser<'i, '_>) -> Result<Self, ParseError<'i>> {
+        parser
+            .try_parse(|p| {
+                Ok(parse_identifiers!(
+                    p,
+                    "SourceGraphic" => Input::SourceGraphic,
+                    "SourceAlpha" => Input::SourceAlpha,
+                    "BackgroundImage" => Input::BackgroundImage,
+                    "BackgroundAlpha" => Input::BackgroundAlpha,
+                    "FillPaint" => Input::FillPaint,
+                    "StrokePaint" => Input::StrokePaint,
+                )?)
+            })
+            .or_else(|_: BasicParseError| {
+                let ident = CustomIdent::parse(parser)?;
+                Ok(Input::FilterOutput(ident))
+            })
+    }
+}
+
 /// The base node for filter primitives which accept input.
 struct PrimitiveWithInput {
     base: Primitive,
@@ -105,26 +130,24 @@ impl Primitive {
         }
     }
 
-    /// Returns the `BoundsBuilder` for bounds computation.
+    /// Validates attributes and returns the `BoundsBuilder` for bounds computation.
     #[inline]
-    fn get_bounds<'a>(&self, ctx: &'a FilterContext) -> BoundsBuilder<'a> {
-        BoundsBuilder::new(ctx, self.x, self.y, self.width, self.height)
-    }
-}
-
-impl NodeTrait for Primitive {
-    fn set_atts(&mut self, parent: Option<&RsvgNode>, pbag: &PropertyBag<'_>) -> NodeResult {
-        // With ObjectBoundingBox, only fractions and percents are allowed.
+    fn get_bounds<'a>(
+        &self,
+        ctx: &'a FilterContext,
+        parent: Option<&Node>,
+    ) -> Result<BoundsBuilder<'a>, FilterError> {
         let primitiveunits = parent
             .and_then(|parent| {
-                if parent.borrow().get_type() == NodeType::Filter {
-                    Some(parent.borrow().get_impl::<Filter>().get_primitive_units())
-                } else {
-                    None
+                assert!(parent.is_element());
+                match *parent.borrow_element() {
+                    Element::Filter(ref f) => Some(f.get_primitive_units()),
+                    _ => None,
                 }
             })
             .unwrap_or(CoordUnits::UserSpaceOnUse);
 
+        // With ObjectBoundingBox, only fractions and percents are allowed.
         let no_units_allowed = primitiveunits == CoordUnits::ObjectBoundingBox;
 
         let check_units_horizontal = |length: Length<Horizontal>| {
@@ -134,9 +157,7 @@ impl NodeTrait for Primitive {
 
             match length.unit {
                 LengthUnit::Px | LengthUnit::Percent => Ok(length),
-                _ => Err(ValueErrorKind::parse_error(
-                    "unit identifiers are not allowed with primitiveUnits set to objectBoundingBox",
-                )),
+                _ => Err(FilterError::InvalidUnits),
             }
         };
 
@@ -147,41 +168,50 @@ impl NodeTrait for Primitive {
 
             match length.unit {
                 LengthUnit::Px | LengthUnit::Percent => Ok(length),
-                _ => Err(ValueErrorKind::parse_error(
-                    "unit identifiers are not allowed with primitiveUnits set to objectBoundingBox",
-                )),
+                _ => Err(FilterError::InvalidUnits),
             }
         };
 
-        let check_units_horizontal_and_ensure_nonnegative = |length: Length<Horizontal>| {
-            check_units_horizontal(length).and_then(Length::<Horizontal>::check_nonnegative)
-        };
+        if let Some(x) = self.x {
+            check_units_horizontal(x)?;
+        }
 
-        let check_units_vertical_and_ensure_nonnegative = |length: Length<Vertical>| {
-            check_units_vertical(length).and_then(Length::<Vertical>::check_nonnegative)
-        };
+        if let Some(y) = self.y {
+            check_units_vertical(y)?;
+        }
 
+        if let Some(w) = self.width {
+            check_units_horizontal(w)?;
+        }
+
+        if let Some(h) = self.height {
+            check_units_vertical(h)?;
+        }
+
+        Ok(BoundsBuilder::new(
+            ctx,
+            self.x,
+            self.y,
+            self.width,
+            self.height,
+        ))
+    }
+}
+
+impl SetAttributes for Primitive {
+    fn set_attributes(&mut self, pbag: &PropertyBag<'_>) -> ElementResult {
         for (attr, value) in pbag.iter() {
             match attr.expanded() {
-                expanded_name!("", "x") => {
-                    self.x = Some(attr.parse_and_validate(value, check_units_horizontal)?)
-                }
-                expanded_name!("", "y") => {
-                    self.y = Some(attr.parse_and_validate(value, check_units_vertical)?)
-                }
+                expanded_name!("", "x") => self.x = Some(attr.parse(value)?),
+                expanded_name!("", "y") => self.y = Some(attr.parse(value)?),
                 expanded_name!("", "width") => {
-                    self.width =
-                        Some(attr.parse_and_validate(
-                            value,
-                            check_units_horizontal_and_ensure_nonnegative,
-                        )?)
+                    self.width = Some(
+                        attr.parse_and_validate(value, Length::<Horizontal>::check_nonnegative)?,
+                    )
                 }
                 expanded_name!("", "height") => {
                     self.height =
-                        Some(attr.parse_and_validate(
-                            value,
-                            check_units_vertical_and_ensure_nonnegative,
-                        )?)
+                        Some(attr.parse_and_validate(value, Length::<Vertical>::check_nonnegative)?)
                 }
                 expanded_name!("", "result") => self.result = Some(attr.parse(value)?),
                 _ => (),
@@ -214,9 +244,9 @@ impl PrimitiveWithInput {
     }
 }
 
-impl NodeTrait for PrimitiveWithInput {
-    fn set_atts(&mut self, parent: Option<&RsvgNode>, pbag: &PropertyBag<'_>) -> NodeResult {
-        self.base.set_atts(parent, pbag)?;
+impl SetAttributes for PrimitiveWithInput {
+    fn set_attributes(&mut self, pbag: &PropertyBag<'_>) -> ElementResult {
+        self.base.set_attributes(pbag)?;
 
         for (attr, value) in pbag.iter() {
             match attr.expanded() {
@@ -240,7 +270,7 @@ impl Deref for PrimitiveWithInput {
 
 /// Applies a filter and returns the resulting surface.
 pub fn render(
-    filter_node: &RsvgNode,
+    filter_node: &Node,
     computed_from_node_being_filtered: &ComputedValues,
     source_surface: SharedImageSurface,
     acquired_nodes: &mut AcquiredNodes,
@@ -248,8 +278,11 @@ pub fn render(
     node_bbox: BoundingBox,
 ) -> Result<SharedImageSurface, RenderingError> {
     let filter_node = &*filter_node;
-    assert_eq!(filter_node.borrow().get_type(), NodeType::Filter);
-    assert!(!filter_node.borrow().is_in_error());
+    assert!(is_element_of_type!(filter_node, Filter));
+
+    if filter_node.borrow_element().is_in_error() {
+        return Ok(source_surface);
+    }
 
     let mut filter_ctx = FilterContext::new(
         filter_node,
@@ -267,9 +300,10 @@ pub fn render(
 
     let primitives = filter_node
         .children()
+        .filter(|c| c.is_element())
         // Skip nodes in error.
         .filter(|c| {
-            let in_error = c.borrow().is_in_error();
+            let in_error = c.borrow_element().is_in_error();
 
             if in_error {
                 rsvg_log!("(ignoring filter primitive {} because it is in error)", c);
@@ -278,22 +312,22 @@ pub fn render(
             !in_error
         })
         // Keep only filter primitives (those that implement the Filter trait)
-        .filter(|c| c.borrow().get_node_trait().as_filter_effect().is_some())
+        .filter(|c| c.borrow_element().as_filter_effect().is_some())
         // Check if the node wants linear RGB.
         .map(|c| {
             let linear_rgb = {
                 let cascaded = CascadedValues::new_from_node(&c);
                 let values = cascaded.get();
 
-                values.color_interpolation_filters == ColorInterpolationFilters::LinearRgb
+                values.color_interpolation_filters() == ColorInterpolationFilters::LinearRgb
             };
 
             (c, linear_rgb)
         });
 
     for (c, linear_rgb) in primitives {
-        let node_data = c.borrow();
-        let filter = node_data.get_node_trait().as_filter_effect().unwrap();
+        let elt = c.borrow_element();
+        let filter = elt.as_filter_effect().unwrap();
 
         let mut render = |filter_ctx: &mut FilterContext| {
             if let Err(err) = filter
