@@ -3,11 +3,14 @@
 use cssparser::{
     self, BasicParseErrorKind, DeclarationListParser, ParseErrorKind, Parser, ParserInput, ToCss,
 };
-use markup5ever::{expanded_name, local_name, namespace_url, ns, QualName};
+use markup5ever::{
+    expanded_name, local_name, namespace_url, ns, ExpandedName, LocalName, QualName,
+};
 use std::collections::HashSet;
 
 use crate::css::{DeclParser, Declaration, Origin};
 use crate::error::*;
+use crate::font_props::*;
 use crate::parsers::{Parse, ParseValue};
 use crate::property_bag::PropertyBag;
 use crate::property_defs::*;
@@ -125,6 +128,12 @@ macro_rules! make_properties {
             $($long_str:tt => $long_field:ident: $long_name:ident,)+
         }
 
+        // These are for when expanded_name!("" "foo") is not defined yet
+        // in markup5ever.  We create an ExpandedName by hand in that case.
+        longhands_not_supported_by_markup5ever: {
+            $($long_m5e_str:tt => $long_m5e_field:ident: $long_m5e_name:ident,)+
+        }
+
         non_properties: {
             $($nonprop_field:ident: $nonprop_name:ident,)+
         }
@@ -139,6 +148,7 @@ macro_rules! make_properties {
         enum PropertyId {
             $($short_name,)+
             $($long_name,)+
+            $($long_m5e_name,)+
             $($nonprop_name,)+
 
             UnsetProperty,
@@ -159,6 +169,7 @@ macro_rules! make_properties {
             // we put all the properties here; these are for SpecifiedValues
             $($short_name(SpecifiedValue<$short_name>),)+
             $($long_name(SpecifiedValue<$long_name>),)+
+            $($long_m5e_name(SpecifiedValue<$long_m5e_name>),)+
             $($nonprop_name(SpecifiedValue<$nonprop_name>),)+
         }
 
@@ -167,6 +178,10 @@ macro_rules! make_properties {
                 $long_name($long_name),
             )+
 
+            $(
+                $long_m5e_name($long_m5e_name),
+            )+
+
             $(
                 $nonprop_name($nonprop_name),
             )+
@@ -178,6 +193,10 @@ macro_rules! make_properties {
                 $long_field: $long_name,
             )+
 
+            $(
+                $long_m5e_field: $long_m5e_name,
+            )+
+
             $(
                 $nonprop_field: $nonprop_name,
             )+
@@ -187,6 +206,7 @@ macro_rules! make_properties {
             fn get_property_id(&self) -> PropertyId {
                 match *self {
                     $(ParsedProperty::$long_name(_) => PropertyId::$long_name,)+
+                    $(ParsedProperty::$long_m5e_name(_) => PropertyId::$long_m5e_name,)+
                     $(ParsedProperty::$short_name(_) => PropertyId::$short_name,)+
                     $(ParsedProperty::$nonprop_name(_) => PropertyId::$nonprop_name,)+
                 }
@@ -197,6 +217,7 @@ macro_rules! make_properties {
 
                 match id {
                     $(PropertyId::$long_name => ParsedProperty::$long_name(Unspecified),)+
+                    $(PropertyId::$long_m5e_name => ParsedProperty::$long_m5e_name(Unspecified),)+
                     $(PropertyId::$short_name => ParsedProperty::$short_name(Unspecified),)+
                     $(PropertyId::$nonprop_name => ParsedProperty::$nonprop_name(Unspecified),)+
 
@@ -216,6 +237,16 @@ macro_rules! make_properties {
                 }
             )+
 
+            $(
+                pub fn $long_m5e_field(&self) -> $long_m5e_name {
+                    if let ComputedValue::$long_m5e_name(v) = self.get_value(PropertyId::$long_m5e_name) {
+                        v
+                    } else {
+                        unreachable!();
+                    }
+                }
+            )+
+
             $(
                 pub fn $nonprop_field(&self) -> $nonprop_name {
                     if let ComputedValue::$nonprop_name(v) = self.get_value(PropertyId::$nonprop_name) {
@@ -229,6 +260,7 @@ macro_rules! make_properties {
             fn set_value(&mut self, computed: ComputedValue) {
                 match computed {
                     $(ComputedValue::$long_name(v) => self.$long_field = v,)+
+                    $(ComputedValue::$long_m5e_name(v) => self.$long_m5e_field = v,)+
                     $(ComputedValue::$nonprop_name(v) => self.$nonprop_field = v,)+
                 }
             }
@@ -241,6 +273,10 @@ macro_rules! make_properties {
                         PropertyId::$long_name =>
                             ComputedValue::$long_name(self.$long_field.clone()),
                     )+
+                    $(
+                        PropertyId::$long_m5e_name =>
+                            ComputedValue::$long_m5e_name(self.$long_m5e_field.clone()),
+                    )+
                     $(
                         PropertyId::$nonprop_name =>
                             ComputedValue::$nonprop_name(self.$nonprop_field.clone()),
@@ -261,6 +297,14 @@ macro_rules! make_properties {
                         Ok(ParsedProperty::$long_name(parse_input(input)?)),
                 )+
 
+                $(
+                    e if e == ExpandedName {
+                        ns: &ns!(),
+                        local: &LocalName::from($long_m5e_str),
+                    } =>
+                        Ok(ParsedProperty::$long_m5e_name(parse_input(input)?)),
+                )+
+
                 $(
                     expanded_name!("", $short_str) => {
                         if accept_shorthands {
@@ -284,7 +328,8 @@ macro_rules! make_properties {
 #[rustfmt::skip]
 make_properties! {
     shorthands: {
-        "marker" => marker: Marker,
+        "font"                        => font                        : Font,
+        "marker"                      => marker                      : Marker,
     }
 
     longhands: {
@@ -335,6 +380,10 @@ make_properties! {
         "writing-mode"                => writing_mode                : WritingMode,
     }
 
+    longhands_not_supported_by_markup5ever: {
+        "line-height"                 => line_height                 : LineHeight,
+    }
+
     // These are not properties, but presentation attributes.  However,
     // both xml:lang and xml:space *do* inherit.  We are abusing the
     // property inheritance code for these XML-specific attributes.
@@ -381,28 +430,76 @@ impl SpecifiedValues {
     }
 
     fn set_property_expanding_shorthands(&mut self, prop: &ParsedProperty, replace: bool) {
-        use crate::properties as p;
-        use crate::properties::ParsedProperty::*;
+        match *prop {
+            ParsedProperty::Font(SpecifiedValue::Specified(ref f)) => {
+                self.expand_font_shorthand(f, replace)
+            }
+            ParsedProperty::Marker(SpecifiedValue::Specified(ref m)) => {
+                self.expand_marker_shorthand(m, replace)
+            }
 
-        if let Marker(SpecifiedValue::Specified(p::Marker(ref v))) = *prop {
-            // Since "marker" is a shorthand property, we'll just expand it here
-            self.set_property(
-                &MarkerStart(SpecifiedValue::Specified(p::MarkerStart(v.clone()))),
-                replace,
-            );
-            self.set_property(
-                &MarkerMid(SpecifiedValue::Specified(p::MarkerMid(v.clone()))),
-                replace,
-            );
-            self.set_property(
-                &MarkerEnd(SpecifiedValue::Specified(p::MarkerEnd(v.clone()))),
-                replace,
-            );
-        } else {
-            self.set_property(prop, replace);
+            _ => self.set_property(prop, replace),
         }
     }
 
+    fn expand_font_shorthand(&mut self, font: &Font, replace: bool) {
+        let FontSpec {
+            style,
+            variant,
+            weight,
+            stretch,
+            size,
+            line_height,
+            family,
+        } = font.to_font_spec();
+
+        self.set_property(
+            &ParsedProperty::FontStyle(SpecifiedValue::Specified(style)),
+            replace,
+        );
+        self.set_property(
+            &ParsedProperty::FontVariant(SpecifiedValue::Specified(variant)),
+            replace,
+        );
+        self.set_property(
+            &ParsedProperty::FontWeight(SpecifiedValue::Specified(weight)),
+            replace,
+        );
+        self.set_property(
+            &ParsedProperty::FontStretch(SpecifiedValue::Specified(stretch)),
+            replace,
+        );
+        self.set_property(
+            &ParsedProperty::FontSize(SpecifiedValue::Specified(size)),
+            replace,
+        );
+        self.set_property(
+            &ParsedProperty::LineHeight(SpecifiedValue::Specified(line_height)),
+            replace,
+        );
+        self.set_property(
+            &ParsedProperty::FontFamily(SpecifiedValue::Specified(family)),
+            replace,
+        );
+    }
+
+    fn expand_marker_shorthand(&mut self, marker: &Marker, replace: bool) {
+        let Marker(v) = marker;
+
+        self.set_property(
+            &ParsedProperty::MarkerStart(SpecifiedValue::Specified(MarkerStart(v.clone()))),
+            replace,
+        );
+        self.set_property(
+            &ParsedProperty::MarkerMid(SpecifiedValue::Specified(MarkerMid(v.clone()))),
+            replace,
+        );
+        self.set_property(
+            &ParsedProperty::MarkerEnd(SpecifiedValue::Specified(MarkerEnd(v.clone()))),
+            replace,
+        );
+    }
+
     pub fn set_parsed_property(&mut self, prop: &ParsedProperty) {
         self.set_property_expanding_shorthands(prop, true);
     }
