@@ -4,7 +4,7 @@ use cssparser::{Parser, Token};
 
 use crate::dasharray::Dasharray;
 use crate::error::*;
-use crate::font_props::{FontSizeSpec, FontWeightSpec, LetterSpacingSpec, MultiFontFamily};
+use crate::font_props::{Font, FontFamily, FontSize, FontWeight, LetterSpacing, LineHeight};
 use crate::iri::IRI;
 use crate::length::*;
 use crate::paint_server::PaintServer;
@@ -26,7 +26,7 @@ make_property!(
             }
 
             fn compute(&self, v: &ComputedValues) -> Self {
-                let font_size = v.font_size().0.value();
+                let font_size = v.font_size().value();
                 let parent = v.baseline_shift();
 
                 match (self.0.unit, parent.0.unit) {
@@ -230,21 +230,27 @@ make_property!(
     newtype_parse: UnitInterval,
 );
 
+// https://drafts.csswg.org/css-fonts-4/#font-prop
+make_property!(
+    ComputedValues,
+    Font,
+    default: Font::Spec(Default::default()),
+    inherits_automatically: true,
+);
+
 // https://www.w3.org/TR/SVG/text.html#FontFamilyProperty
 make_property!(
     ComputedValues,
     FontFamily,
-    default: MultiFontFamily("Times New Roman".to_string()),
+    default: FontFamily("Times New Roman".to_string()),
     inherits_automatically: true,
-    newtype_parse: MultiFontFamily,
 );
 
 // https://www.w3.org/TR/SVG/text.html#FontSizeProperty
 make_property!(
     ComputedValues,
     FontSize,
-    default: FontSizeSpec::Value(Length::<Both>::parse_str("12.0").unwrap()),
-    newtype_parse: FontSizeSpec,
+    default: FontSize::Value(Length::<Both>::parse_str("12.0").unwrap()),
     property_impl: {
         impl Property<ComputedValues> for FontSize {
             fn inherits_automatically() -> bool {
@@ -252,7 +258,7 @@ make_property!(
             }
 
             fn compute(&self, v: &ComputedValues) -> Self {
-                FontSize(self.0.compute(v))
+                self.compute(v)
             }
         }
     }
@@ -304,21 +310,29 @@ make_property!(
     "small-caps" => SmallCaps,
 );
 
-// https://www.w3.org/TR/2008/REC-CSS2-20080411/fonts.html#propdef-font-weight
+// https://drafts.csswg.org/css-fonts-4/#font-weight-prop
 make_property!(
     ComputedValues,
     FontWeight,
-    default: FontWeightSpec::Normal,
-    inherits_automatically: true,
-    newtype_parse: FontWeightSpec,
+    default: FontWeight::Normal,
+    property_impl: {
+        impl Property<ComputedValues> for FontWeight {
+            fn inherits_automatically() -> bool {
+                true
+            }
+
+            fn compute(&self, v: &ComputedValues) -> Self {
+                self.compute(&v.font_weight())
+            }
+        }
+    }
 );
 
 // https://www.w3.org/TR/SVG/text.html#LetterSpacingProperty
 make_property!(
     ComputedValues,
     LetterSpacing,
-    default: LetterSpacingSpec::Normal,
-    newtype_parse: LetterSpacingSpec,
+    default: LetterSpacing::Normal,
     property_impl: {
         impl Property<ComputedValues> for LetterSpacing {
             fn inherits_automatically() -> bool {
@@ -326,12 +340,20 @@ make_property!(
             }
 
             fn compute(&self, _v: &ComputedValues) -> Self {
-                LetterSpacing(self.0.compute())
+                self.compute()
             }
         }
     }
 );
 
+// https://drafts.csswg.org/css2/visudet.html#propdef-line-height
+make_property!(
+    ComputedValues,
+    LineHeight,
+    default: LineHeight::Normal,
+    inherits_automatically: true,
+);
+
 // https://www.w3.org/TR/SVG/filters.html#LightingColorProperty
 make_property!(
     ComputedValues,
