@@ -8,11 +8,12 @@ use std::rc::Rc;
 use crate::bbox::BoundingBox;
 use crate::document::AcquiredNodes;
 use crate::drawing_ctx::DrawingCtx;
+use crate::element::{Draw, ElementResult, SetAttributes};
 use crate::error::*;
 use crate::length::*;
-use crate::node::*;
+use crate::node::{CascadedValues, Node};
 use crate::parsers::{optional_comma, Parse, ParseValue};
-use crate::path_builder::*;
+use crate::path_builder::{LargeArc, Path as SvgPath, PathBuilder, Sweep};
 use crate::path_parser;
 use crate::properties::ComputedValues;
 use crate::property_bag::PropertyBag;
@@ -25,25 +26,25 @@ pub enum Markers {
 }
 
 pub struct Shape {
-    builder: Rc<PathBuilder>,
+    path: Rc<SvgPath>,
     markers: Markers,
 }
 
 impl Shape {
-    fn new(builder: Rc<PathBuilder>, markers: Markers) -> Shape {
-        Shape { builder, markers }
+    fn new(path: Rc<SvgPath>, markers: Markers) -> Shape {
+        Shape { path, markers }
     }
 
     fn draw(
         &self,
-        node: &RsvgNode,
+        node: &Node,
         acquired_nodes: &mut AcquiredNodes,
         values: &ComputedValues,
         draw_ctx: &mut DrawingCtx,
         clipping: bool,
     ) -> Result<BoundingBox, RenderingError> {
         draw_ctx.draw_path(
-            &self.builder,
+            &self.path,
             node,
             acquired_nodes,
             values,
@@ -53,12 +54,12 @@ impl Shape {
     }
 }
 
-fn make_ellipse(cx: f64, cy: f64, rx: f64, ry: f64) -> PathBuilder {
+fn make_ellipse(cx: f64, cy: f64, rx: f64, ry: f64) -> SvgPath {
     let mut builder = PathBuilder::new();
 
     // Per the spec, rx and ry must be nonnegative
     if rx <= 0.0 || ry <= 0.0 {
-        return builder;
+        return builder.into_path();
     }
 
     // 4/3 * (1-cos 45°)/sin 45° = 4/3 * sqrt(2) - 1
@@ -106,16 +107,16 @@ fn make_ellipse(cx: f64, cy: f64, rx: f64, ry: f64) -> PathBuilder {
 
     builder.close_path();
 
-    builder
+    builder.into_path()
 }
 
 #[derive(Default)]
 pub struct Path {
-    builder: Option<Rc<PathBuilder>>,
+    path: Option<Rc<SvgPath>>,
 }
 
-impl NodeTrait for Path {
-    fn set_atts(&mut self, _: Option<&RsvgNode>, pbag: &PropertyBag<'_>) -> NodeResult {
+impl SetAttributes for Path {
+    fn set_attributes(&mut self, pbag: &PropertyBag<'_>) -> ElementResult {
         for (attr, value) in pbag.iter() {
             if attr.expanded() == expanded_name!("", "d") {
                 let mut builder = PathBuilder::new();
@@ -125,24 +126,26 @@ impl NodeTrait for Path {
 
                     rsvg_log!("could not parse path: {}", e);
                 }
-                self.builder = Some(Rc::new(builder));
+                self.path = Some(Rc::new(builder.into_path()));
             }
         }
 
         Ok(())
     }
+}
 
+impl Draw for Path {
     fn draw(
         &self,
-        node: &RsvgNode,
+        node: &Node,
         acquired_nodes: &mut AcquiredNodes,
         cascaded: &CascadedValues<'_>,
         draw_ctx: &mut DrawingCtx,
         clipping: bool,
     ) -> Result<BoundingBox, RenderingError> {
-        if let Some(builder) = self.builder.as_ref() {
+        if let Some(path) = self.path.as_ref() {
             let values = cascaded.get();
-            Shape::new(builder.clone(), Markers::Yes).draw(
+            Shape::new(path.clone(), Markers::Yes).draw(
                 node,
                 acquired_nodes,
                 values,
@@ -193,7 +196,7 @@ impl Parse for Points {
     }
 }
 
-fn make_poly(points: Option<&Points>, closed: bool) -> PathBuilder {
+fn make_poly(points: Option<&Points>, closed: bool) -> SvgPath {
     let mut builder = PathBuilder::new();
 
     if let Some(points) = points {
@@ -210,7 +213,7 @@ fn make_poly(points: Option<&Points>, closed: bool) -> PathBuilder {
         }
     }
 
-    builder
+    builder.into_path()
 }
 
 #[derive(Default)]
@@ -218,8 +221,8 @@ pub struct Polygon {
     points: Option<Points>,
 }
 
-impl NodeTrait for Polygon {
-    fn set_atts(&mut self, _: Option<&RsvgNode>, pbag: &PropertyBag<'_>) -> NodeResult {
+impl SetAttributes for Polygon {
+    fn set_attributes(&mut self, pbag: &PropertyBag<'_>) -> ElementResult {
         for (attr, value) in pbag.iter() {
             if attr.expanded() == expanded_name!("", "points") {
                 self.points = attr.parse(value).map(Some)?;
@@ -228,10 +231,12 @@ impl NodeTrait for Polygon {
 
         Ok(())
     }
+}
 
+impl Draw for Polygon {
     fn draw(
         &self,
-        node: &RsvgNode,
+        node: &Node,
         acquired_nodes: &mut AcquiredNodes,
         cascaded: &CascadedValues<'_>,
         draw_ctx: &mut DrawingCtx,
@@ -253,8 +258,8 @@ pub struct Polyline {
     points: Option<Points>,
 }
 
-impl NodeTrait for Polyline {
-    fn set_atts(&mut self, _: Option<&RsvgNode>, pbag: &PropertyBag<'_>) -> NodeResult {
+impl SetAttributes for Polyline {
+    fn set_attributes(&mut self, pbag: &PropertyBag<'_>) -> ElementResult {
         for (attr, value) in pbag.iter() {
             if attr.expanded() == expanded_name!("", "points") {
                 self.points = attr.parse(value).map(Some)?;
@@ -263,10 +268,12 @@ impl NodeTrait for Polyline {
 
         Ok(())
     }
+}
 
+impl Draw for Polyline {
     fn draw(
         &self,
-        node: &RsvgNode,
+        node: &Node,
         acquired_nodes: &mut AcquiredNodes,
         cascaded: &CascadedValues<'_>,
         draw_ctx: &mut DrawingCtx,
@@ -289,8 +296,8 @@ pub struct Line {
     y2: Length<Vertical>,
 }
 
-impl NodeTrait for Line {
-    fn set_atts(&mut self, _: Option<&RsvgNode>, pbag: &PropertyBag<'_>) -> NodeResult {
+impl SetAttributes for Line {
+    fn set_attributes(&mut self, pbag: &PropertyBag<'_>) -> ElementResult {
         for (attr, value) in pbag.iter() {
             match attr.expanded() {
                 expanded_name!("", "x1") => self.x1 = attr.parse(value)?,
@@ -303,26 +310,30 @@ impl NodeTrait for Line {
 
         Ok(())
     }
+}
 
+impl Draw for Line {
     fn draw(
         &self,
-        node: &RsvgNode,
+        node: &Node,
         acquired_nodes: &mut AcquiredNodes,
         cascaded: &CascadedValues<'_>,
         draw_ctx: &mut DrawingCtx,
         clipping: bool,
     ) -> Result<BoundingBox, RenderingError> {
         let values = cascaded.get();
-        Shape::new(
-            Rc::new(self.make_path_builder(values, draw_ctx)),
-            Markers::Yes,
+        Shape::new(Rc::new(self.make_path(values, draw_ctx)), Markers::Yes).draw(
+            node,
+            acquired_nodes,
+            values,
+            draw_ctx,
+            clipping,
         )
-        .draw(node, acquired_nodes, values, draw_ctx, clipping)
     }
 }
 
 impl Line {
-    fn make_path_builder(&self, values: &ComputedValues, draw_ctx: &mut DrawingCtx) -> PathBuilder {
+    fn make_path(&self, values: &ComputedValues, draw_ctx: &mut DrawingCtx) -> SvgPath {
         let mut builder = PathBuilder::new();
 
         let params = draw_ctx.get_view_params();
@@ -335,7 +346,7 @@ impl Line {
         builder.move_to(x1, y1);
         builder.line_to(x2, y2);
 
-        builder
+        builder.into_path()
     }
 }
 
@@ -351,8 +362,8 @@ pub struct Rect {
     ry: Option<Length<Vertical>>,
 }
 
-impl NodeTrait for Rect {
-    fn set_atts(&mut self, _: Option<&RsvgNode>, pbag: &PropertyBag<'_>) -> NodeResult {
+impl SetAttributes for Rect {
+    fn set_attributes(&mut self, pbag: &PropertyBag<'_>) -> ElementResult {
         for (attr, value) in pbag.iter() {
             match attr.expanded() {
                 expanded_name!("", "x") => self.x = attr.parse(value)?,
@@ -381,26 +392,30 @@ impl NodeTrait for Rect {
 
         Ok(())
     }
+}
 
+impl Draw for Rect {
     fn draw(
         &self,
-        node: &RsvgNode,
+        node: &Node,
         acquired_nodes: &mut AcquiredNodes,
         cascaded: &CascadedValues<'_>,
         draw_ctx: &mut DrawingCtx,
         clipping: bool,
     ) -> Result<BoundingBox, RenderingError> {
         let values = cascaded.get();
-        Shape::new(
-            Rc::new(self.make_path_builder(values, draw_ctx)),
-            Markers::No,
+        Shape::new(Rc::new(self.make_path(values, draw_ctx)), Markers::No).draw(
+            node,
+            acquired_nodes,
+            values,
+            draw_ctx,
+            clipping,
         )
-        .draw(node, acquired_nodes, values, draw_ctx, clipping)
     }
 }
 
 impl Rect {
-    fn make_path_builder(&self, values: &ComputedValues, draw_ctx: &mut DrawingCtx) -> PathBuilder {
+    fn make_path(&self, values: &ComputedValues, draw_ctx: &mut DrawingCtx) -> SvgPath {
         let params = draw_ctx.get_view_params();
 
         let x = self.x.normalize(values, &params);
@@ -437,12 +452,12 @@ impl Rect {
 
         // Per the spec, w,h must be >= 0
         if w <= 0.0 || h <= 0.0 {
-            return builder;
+            return builder.into_path();
         }
 
         // ... and rx,ry must be nonnegative
         if rx < 0.0 || ry < 0.0 {
-            return builder;
+            return builder.into_path();
         }
 
         let half_w = w / 2.0;
@@ -568,7 +583,7 @@ impl Rect {
             builder.close_path();
         }
 
-        builder
+        builder.into_path()
     }
 }
 
@@ -579,8 +594,8 @@ pub struct Circle {
     r: Length<Both>,
 }
 
-impl NodeTrait for Circle {
-    fn set_atts(&mut self, _: Option<&RsvgNode>, pbag: &PropertyBag<'_>) -> NodeResult {
+impl SetAttributes for Circle {
+    fn set_attributes(&mut self, pbag: &PropertyBag<'_>) -> ElementResult {
         for (attr, value) in pbag.iter() {
             match attr.expanded() {
                 expanded_name!("", "cx") => self.cx = attr.parse(value)?,
@@ -594,26 +609,30 @@ impl NodeTrait for Circle {
 
         Ok(())
     }
+}
 
+impl Draw for Circle {
     fn draw(
         &self,
-        node: &RsvgNode,
+        node: &Node,
         acquired_nodes: &mut AcquiredNodes,
         cascaded: &CascadedValues<'_>,
         draw_ctx: &mut DrawingCtx,
         clipping: bool,
     ) -> Result<BoundingBox, RenderingError> {
         let values = cascaded.get();
-        Shape::new(
-            Rc::new(self.make_path_builder(values, draw_ctx)),
-            Markers::No,
+        Shape::new(Rc::new(self.make_path(values, draw_ctx)), Markers::No).draw(
+            node,
+            acquired_nodes,
+            values,
+            draw_ctx,
+            clipping,
         )
-        .draw(node, acquired_nodes, values, draw_ctx, clipping)
     }
 }
 
 impl Circle {
-    fn make_path_builder(&self, values: &ComputedValues, draw_ctx: &mut DrawingCtx) -> PathBuilder {
+    fn make_path(&self, values: &ComputedValues, draw_ctx: &mut DrawingCtx) -> SvgPath {
         let params = draw_ctx.get_view_params();
 
         let cx = self.cx.normalize(values, &params);
@@ -632,8 +651,8 @@ pub struct Ellipse {
     ry: Length<Vertical>,
 }
 
-impl NodeTrait for Ellipse {
-    fn set_atts(&mut self, _: Option<&RsvgNode>, pbag: &PropertyBag<'_>) -> NodeResult {
+impl SetAttributes for Ellipse {
+    fn set_attributes(&mut self, pbag: &PropertyBag<'_>) -> ElementResult {
         for (attr, value) in pbag.iter() {
             match attr.expanded() {
                 expanded_name!("", "cx") => self.cx = attr.parse(value)?,
@@ -652,26 +671,30 @@ impl NodeTrait for Ellipse {
 
         Ok(())
     }
+}
 
+impl Draw for Ellipse {
     fn draw(
         &self,
-        node: &RsvgNode,
+        node: &Node,
         acquired_nodes: &mut AcquiredNodes,
         cascaded: &CascadedValues<'_>,
         draw_ctx: &mut DrawingCtx,
         clipping: bool,
     ) -> Result<BoundingBox, RenderingError> {
         let values = cascaded.get();
-        Shape::new(
-            Rc::new(self.make_path_builder(values, draw_ctx)),
-            Markers::No,
+        Shape::new(Rc::new(self.make_path(values, draw_ctx)), Markers::No).draw(
+            node,
+            acquired_nodes,
+            values,
+            draw_ctx,
+            clipping,
         )
-        .draw(node, acquired_nodes, values, draw_ctx, clipping)
     }
 }
 
 impl Ellipse {
-    fn make_path_builder(&self, values: &ComputedValues, draw_ctx: &mut DrawingCtx) -> PathBuilder {
+    fn make_path(&self, values: &ComputedValues, draw_ctx: &mut DrawingCtx) -> SvgPath {
         let params = draw_ctx.get_view_params();
 
         let cx = self.cx.normalize(values, &params);
