@@ -9,9 +9,10 @@ use crate::coord_units::CoordUnits;
 use crate::document::AcquiredNodes;
 use crate::dpi::Dpi;
 use crate::drawing_ctx::{ClipMode, DrawingCtx, ViewParams};
+use crate::element::{Draw, ElementResult, SetAttributes};
 use crate::error::*;
 use crate::length::*;
-use crate::node::*;
+use crate::node::{CascadedValues, Node, NodeBorrow, NodeDraw};
 use crate::parsers::{Parse, ParseValue};
 use crate::properties::ComputedValues;
 use crate::property_bag::PropertyBag;
@@ -21,14 +22,12 @@ use crate::viewbox::*;
 #[derive(Default)]
 pub struct Group();
 
-impl NodeTrait for Group {
-    fn set_atts(&mut self, _: Option<&RsvgNode>, _: &PropertyBag<'_>) -> NodeResult {
-        Ok(())
-    }
+impl SetAttributes for Group {}
 
+impl Draw for Group {
     fn draw(
         &self,
-        node: &RsvgNode,
+        node: &Node,
         acquired_nodes: &mut AcquiredNodes,
         cascaded: &CascadedValues<'_>,
         draw_ctx: &mut DrawingCtx,
@@ -49,23 +48,19 @@ impl NodeTrait for Group {
 #[derive(Default)]
 pub struct NonRendering;
 
-impl NodeTrait for NonRendering {
-    fn set_atts(&mut self, _: Option<&RsvgNode>, _: &PropertyBag<'_>) -> NodeResult {
-        Ok(())
-    }
-}
+impl SetAttributes for NonRendering {}
+
+impl Draw for NonRendering {}
 
 #[derive(Default)]
 pub struct Switch();
 
-impl NodeTrait for Switch {
-    fn set_atts(&mut self, _: Option<&RsvgNode>, _: &PropertyBag<'_>) -> NodeResult {
-        Ok(())
-    }
+impl SetAttributes for Switch {}
 
+impl Draw for Switch {
     fn draw(
         &self,
-        node: &RsvgNode,
+        node: &Node,
         acquired_nodes: &mut AcquiredNodes,
         cascaded: &CascadedValues<'_>,
         draw_ctx: &mut DrawingCtx,
@@ -76,8 +71,8 @@ impl NodeTrait for Switch {
         draw_ctx.with_discrete_layer(node, acquired_nodes, values, clipping, &mut |an, dc| {
             if let Some(child) = node
                 .children()
-                .filter(|c| c.borrow().get_type() != NodeType::Chars)
-                .find(|c| c.borrow().get_cond())
+                .filter(|c| c.is_element())
+                .find(|c| c.borrow_element().get_cond())
             {
                 dc.draw_node_from_stack(
                     &child,
@@ -92,11 +87,20 @@ impl NodeTrait for Switch {
     }
 }
 
-/// Intrinsic dimensions of an SVG document fragment
+/// Intrinsic dimensions of an SVG document fragment: its `width`, `height`, `viewBox` attributes.
+///
+/// Note that either of those attributes can be omitted, so they are all `Option<T>`.
+/// For example, an element like `<svg viewBox="0 0 100 100">` will have `vbox=Some(...)`,
+/// and the other two fields set to `None`.
 #[derive(Debug, Copy, Clone, PartialEq)]
 pub struct IntrinsicDimensions {
+    /// Contents of the `width` attribute.
     pub width: Option<Length<Horizontal>>,
+
+    /// Contents of the `height` attribute.
     pub height: Option<Length<Vertical>>,
+
+    /// Contents of the `viewBox` attribute.
     pub vbox: Option<ViewBox>,
 }
 
@@ -111,18 +115,25 @@ pub struct Svg {
 }
 
 impl Svg {
+    /// Returns the SVG's size suitable for the legacy C API, or None
+    /// if it must be computed by hand.
+    ///
+    /// The legacy C API can compute an SVG document's size from the
+    /// `width`, `height`, and `viewBox` attributes of the toplevel `<svg>`
+    /// element.  If these are not available, then the size must be computed
+    /// by actually measuring the geometries of elements in the document.
     pub fn get_size(&self, values: &ComputedValues, dpi: Dpi) -> Option<(f64, f64)> {
-        let (_, _, w, h) = self.get_unnormalized_viewport();
+        let (w, h) = self.get_unnormalized_size();
 
         match (w, h, self.vbox) {
             (w, h, Some(vbox)) => {
-                let params = ViewParams::new(dpi.x(), dpi.y(), vbox.0.width(), vbox.0.height());
+                let params = ViewParams::new(dpi, vbox.0.width(), vbox.0.height());
 
                 Some((w.normalize(values, &params), h.normalize(values, &params)))
             }
 
             (w, h, None) if w.unit != LengthUnit::Percent && h.unit != LengthUnit::Percent => {
-                let params = ViewParams::new(dpi.x(), dpi.y(), 0.0, 0.0);
+                let params = ViewParams::new(dpi, 0.0, 0.0);
 
                 Some((w.normalize(values, &params), h.normalize(values, &params)))
             }
@@ -138,15 +149,7 @@ impl Svg {
         }
     }
 
-    // returns (x, y, w, h)
-    fn get_unnormalized_viewport(
-        &self,
-    ) -> (
-        Length<Horizontal>,
-        Length<Vertical>,
-        Length<Horizontal>,
-        Length<Vertical>,
-    ) {
+    fn get_unnormalized_offset(&self) -> (Length<Horizontal>, Length<Vertical>) {
         // these defaults are per the spec
         let x = self
             .x
@@ -154,6 +157,12 @@ impl Svg {
         let y = self
             .y
             .unwrap_or_else(|| Length::<Vertical>::parse_str("0").unwrap());
+
+        (x, y)
+    }
+
+    fn get_unnormalized_size(&self) -> (Length<Horizontal>, Length<Vertical>) {
+        // these defaults are per the spec
         let w = self
             .w
             .unwrap_or_else(|| Length::<Horizontal>::parse_str("100%").unwrap());
@@ -161,34 +170,35 @@ impl Svg {
             .h
             .unwrap_or_else(|| Length::<Vertical>::parse_str("100%").unwrap());
 
-        (x, y, w, h)
+        (w, h)
     }
 
-    fn get_viewport(&self, values: &ComputedValues, params: &ViewParams) -> Rect {
-        let (x, y, w, h) = self.get_unnormalized_viewport();
+    fn get_viewport(&self, values: &ComputedValues, params: &ViewParams, outermost: bool) -> Rect {
+        // x & y attributes have no effect on outermost svg
+        // http://www.w3.org/TR/SVG/struct.html#SVGElement
+        let (nx, ny) = if outermost {
+            (0.0, 0.0)
+        } else {
+            let (x, y) = self.get_unnormalized_offset();
+            (x.normalize(values, &params), y.normalize(values, &params))
+        };
 
-        let nx = x.normalize(values, &params);
-        let ny = y.normalize(values, &params);
-        let nw = w.normalize(values, &params);
-        let nh = h.normalize(values, &params);
+        let (w, h) = self.get_unnormalized_size();
+        let (nw, nh) = (w.normalize(values, &params), h.normalize(values, &params));
 
         Rect::new(nx, ny, nx + nw, ny + nh)
     }
 }
 
-impl NodeTrait for Svg {
-    fn set_atts(&mut self, parent: Option<&RsvgNode>, pbag: &PropertyBag<'_>) -> NodeResult {
-        // x & y attributes have no effect on outermost svg
-        // http://www.w3.org/TR/SVG/struct.html#SVGElement
-        let is_inner_svg = parent.is_some();
-
+impl SetAttributes for Svg {
+    fn set_attributes(&mut self, pbag: &PropertyBag<'_>) -> ElementResult {
         for (attr, value) in pbag.iter() {
             match attr.expanded() {
                 expanded_name!("", "preserveAspectRatio") => {
                     self.preserve_aspect_ratio = attr.parse(value)?
                 }
-                expanded_name!("", "x") if is_inner_svg => self.x = Some(attr.parse(value)?),
-                expanded_name!("", "y") if is_inner_svg => self.y = Some(attr.parse(value)?),
+                expanded_name!("", "x") => self.x = Some(attr.parse(value)?),
+                expanded_name!("", "y") => self.y = Some(attr.parse(value)?),
                 expanded_name!("", "width") => {
                     self.w = Some(
                         attr.parse_and_validate(value, Length::<Horizontal>::check_nonnegative)?,
@@ -205,14 +215,12 @@ impl NodeTrait for Svg {
 
         Ok(())
     }
+}
 
-    fn overflow_hidden(&self) -> bool {
-        true
-    }
-
+impl Draw for Svg {
     fn draw(
         &self,
-        node: &RsvgNode,
+        node: &Node,
         acquired_nodes: &mut AcquiredNodes,
         cascaded: &CascadedValues<'_>,
         draw_ctx: &mut DrawingCtx,
@@ -230,7 +238,7 @@ impl NodeTrait for Svg {
             None
         };
 
-        let svg_viewport = self.get_viewport(values, &params);
+        let svg_viewport = self.get_viewport(values, &params, !has_parent);
 
         let is_measuring_toplevel_svg = !has_parent && draw_ctx.is_measuring();
 
@@ -275,10 +283,6 @@ pub struct Use {
 }
 
 impl Use {
-    pub fn get_link(&self) -> Option<&Fragment> {
-        self.link.as_ref()
-    }
-
     pub fn get_rect(&self, values: &ComputedValues, params: &ViewParams) -> Rect {
         let x = self.x.normalize(values, &params);
         let y = self.y.normalize(values, &params);
@@ -301,8 +305,8 @@ impl Use {
     }
 }
 
-impl NodeTrait for Use {
-    fn set_atts(&mut self, _: Option<&RsvgNode>, pbag: &PropertyBag<'_>) -> NodeResult {
+impl SetAttributes for Use {
+    fn set_attributes(&mut self, pbag: &PropertyBag<'_>) -> ElementResult {
         for (attr, value) in pbag.iter() {
             match attr.expanded() {
                 expanded_name!(xlink "href") => {
@@ -326,16 +330,18 @@ impl NodeTrait for Use {
 
         Ok(())
     }
+}
 
+impl Draw for Use {
     fn draw(
         &self,
-        node: &RsvgNode,
+        node: &Node,
         acquired_nodes: &mut AcquiredNodes,
         cascaded: &CascadedValues<'_>,
         draw_ctx: &mut DrawingCtx,
         clipping: bool,
     ) -> Result<BoundingBox, RenderingError> {
-        draw_ctx.draw_from_use_node(node, acquired_nodes, cascaded, clipping)
+        draw_ctx.draw_from_use_node(node, acquired_nodes, cascaded, self.link.as_ref(), clipping)
     }
 }
 
@@ -355,8 +361,8 @@ impl Symbol {
     }
 }
 
-impl NodeTrait for Symbol {
-    fn set_atts(&mut self, _parent: Option<&RsvgNode>, pbag: &PropertyBag<'_>) -> NodeResult {
+impl SetAttributes for Symbol {
+    fn set_attributes(&mut self, pbag: &PropertyBag<'_>) -> ElementResult {
         for (attr, value) in pbag.iter() {
             match attr.expanded() {
                 expanded_name!("", "preserveAspectRatio") => {
@@ -369,12 +375,10 @@ impl NodeTrait for Symbol {
 
         Ok(())
     }
-
-    fn overflow_hidden(&self) -> bool {
-        true
-    }
 }
 
+impl Draw for Symbol {}
+
 coord_units!(ClipPathUnits, CoordUnits::UserSpaceOnUse);
 
 #[derive(Default)]
@@ -388,8 +392,8 @@ impl ClipPath {
     }
 }
 
-impl NodeTrait for ClipPath {
-    fn set_atts(&mut self, _: Option<&RsvgNode>, pbag: &PropertyBag<'_>) -> NodeResult {
+impl SetAttributes for ClipPath {
+    fn set_attributes(&mut self, pbag: &PropertyBag<'_>) -> ElementResult {
         for (attr, value) in pbag.iter() {
             match attr.expanded() {
                 expanded_name!("", "clipPathUnits") => self.units = attr.parse(value)?,
@@ -401,6 +405,8 @@ impl NodeTrait for ClipPath {
     }
 }
 
+impl Draw for ClipPath {}
+
 coord_units!(MaskUnits, CoordUnits::ObjectBoundingBox);
 coord_units!(MaskContentUnits, CoordUnits::UserSpaceOnUse);
 
@@ -448,8 +454,8 @@ impl Mask {
     }
 }
 
-impl NodeTrait for Mask {
-    fn set_atts(&mut self, _: Option<&RsvgNode>, pbag: &PropertyBag<'_>) -> NodeResult {
+impl SetAttributes for Mask {
+    fn set_attributes(&mut self, pbag: &PropertyBag<'_>) -> ElementResult {
         for (attr, value) in pbag.iter() {
             match attr.expanded() {
                 expanded_name!("", "x") => self.x = attr.parse(value)?,
@@ -472,13 +478,15 @@ impl NodeTrait for Mask {
     }
 }
 
+impl Draw for Mask {}
+
 #[derive(Default)]
 pub struct Link {
     link: Option<String>,
 }
 
-impl NodeTrait for Link {
-    fn set_atts(&mut self, _: Option<&RsvgNode>, pbag: &PropertyBag<'_>) -> NodeResult {
+impl SetAttributes for Link {
+    fn set_attributes(&mut self, pbag: &PropertyBag<'_>) -> ElementResult {
         for (attr, value) in pbag.iter() {
             match attr.expanded() {
                 expanded_name!(xlink "href") => self.link = Some(value.to_owned()),
@@ -488,10 +496,12 @@ impl NodeTrait for Link {
 
         Ok(())
     }
+}
 
+impl Draw for Link {
     fn draw(
         &self,
-        node: &RsvgNode,
+        node: &Node,
         acquired_nodes: &mut AcquiredNodes,
         cascaded: &CascadedValues<'_>,
         draw_ctx: &mut DrawingCtx,
